#define QT_FEATURE_clock_gettime -1
#define QT_FEATURE_datetimeparser 1
#define QT_FEATURE_dlopen -1
#define QT_FEATURE_doubleconversion 1
#define QT_FEATURE_glib -1
#define QT_FEATURE_gnu_libiconv -1
#define QT_FEATURE_icu 1
#define QT_FEATURE_iconv -1
#define QT_FEATURE_inotify -1
#define QT_FEATURE_journald -1
#define QT_FEATURE_posix_libiconv -1
#define QT_FEATURE_sha3_fast 1
#define QT_FEATURE_slog2 -1
#define QT_FEATURE_syslog -1
#define QT_FEATURE_system_doubleconversion -1
#define QT_FEATURE_system_pcre2 -1
