#define QT_FEATURE_networkinterface 1
#define QT_FEATURE_bearermanagement 1
#define QT_FEATURE_ftp 1
#define QT_NO_GETADDRINFO 
#define QT_NO_GETIFADDRS 
#define QT_FEATURE_http 1
#define QT_NO_IPV6IFNAME 
#define QT_FEATURE_localserver 1
#define QT_FEATURE_networkdiskcache 1
#define QT_FEATURE_networkproxy 1
#define QT_FEATURE_sctp -1
#define QT_NO_SCTP 
#define QT_FEATURE_socks5 1
#define QT_FEATURE_ssl 1
#define QT_FEATURE_udpsocket 1
