/* This file was generated by syncqt. */
#ifndef QT_QTCONCURRENT_VERSION_H
#define QT_QTCONCURRENT_VERSION_H

#define QTCONCURRENT_VERSION_STR "5.9.6"

#define QTCONCURRENT_VERSION 0x050906

#endif // QT_QTCONCURRENT_VERSION_H
